@@ -25,6 +25,7 @@
 #include <qdatastream.h>
 
 #include <math.h>
+#include <GL/glu.h>
 
 #include <lib3ds/mesh.h>
 
@@ -378,7 +379,7 @@ void BoMatrix::rotate(GLfloat angle, GLfloat x, GLfloat y, GLfloat z)
 
  mag = (GLfloat) sqrt( x*x + y*y + z*z ); // AB: mesa uses GL_SQRT here
 
- if (mag <= 1.0e-4) 
+ if (mag <= 1.0e-4)
  {
    // generate an identity matrix and return
    loadIdentity();
@@ -621,7 +622,7 @@ void BoMatrix::extractUp(BoVector3& up, const BoVector3& x, const BoVector3& z)
  up.setY(foo1 / z[2]);
 
  up.setX((up.y() * z[0] + x[2]) / z[1]);
- 
+
 }
 
 
@@ -1147,6 +1148,122 @@ void Bo3dTools::enableReadDepthBufferWorkaround(float _1_0_depthValue)
   workaround_depth_value_enabled = true;
 }
 
+void Bo3dTools::disableReadDepthBufferWorkaround()
+{
+  workaround_depth_value_enabled = false;
+}
+
+bool Bo3dTools::checkError(GLenum* error, QString* errorString, QString* errorName)
+{
+  bool ret = true;
+  GLenum e = glGetError();
+  QString s;
+  switch(e)
+  {
+    case GL_INVALID_ENUM:
+      s =  "GL_INVALID_ENUM";
+      break;
+    case GL_INVALID_VALUE:
+      s = "GL_INVALID_VALUE";
+      break;
+    case GL_INVALID_OPERATION:
+      s = "GL_INVALID_OPERATION";
+      break;
+    case GL_STACK_OVERFLOW:
+      s = "GL_STACK_OVERFLOW";
+      break;
+    case GL_STACK_UNDERFLOW:
+      s = "GL_STACK_UNDERFLOW";
+      break;
+    case GL_OUT_OF_MEMORY:
+      s = "GL_OUT_OF_MEMORY";
+      break;
+    case GL_NO_ERROR:
+      ret = false;
+      break;
+    default:
+      s = QString("Unknown OpenGL Error: %1").arg((int)e);
+      break;
+  }
+  if(!s.isEmpty())
+  {
+    boError() << s << endl;
+  }
+  QString string;
+  if(e != GL_NO_ERROR)
+  {
+    string = QString((char*)gluErrorString(e));
+    boError() << "Error string: " << string << endl;
+  }
+  if(error)
+  {
+    *error = e;
+  }
+  if(errorString)
+  {
+    *errorString = string;
+  }
+  if(errorName)
+  {
+    *errorName = s;
+  }
+  return ret;
+}
+
+GLenum Bo3dTools::string2GLBlendFunc(const QString& str)
+{
+  if(str == "GL_SRC_ALPHA")
+  {
+    return GL_SRC_ALPHA;
+  }
+  else if(str == "GL_ONE_MINUS_SRC_ALPHA")
+  {
+    return GL_ONE_MINUS_SRC_ALPHA;
+  }
+  else if(str == "GL_ONE")
+  {
+    return GL_ONE;
+  }
+  else if(str == "GL_ZERO")
+  {
+    return GL_ZERO;
+  }
+  else if(str == "GL_DST_COLOR")
+  {
+    return GL_DST_COLOR;
+  }
+  else if(str == "GL_ONE_MINUS_DST_COLOR")
+  {
+    return GL_ONE_MINUS_DST_COLOR;
+  }
+  else if(str == "GL_DST_ALPHA")
+  {
+    return GL_DST_ALPHA;
+  }
+  else if(str == "GL_ONE_MINUS_DST_ALPHA")
+  {
+    return GL_ONE_MINUS_DST_ALPHA;
+  }
+  else if(str == "GL_SRC_ALPHA_SATURATE")
+  {
+    return GL_SRC_ALPHA_SATURATE;
+  }
+  else if(str == "GL_SRC_COLOR")
+  {
+    return GL_SRC_COLOR;
+  }
+  else if(str == "GL_ONE_MINUS_SRC_COLOR")
+  {
+    return GL_ONE_MINUS_SRC_COLOR;
+  }
+  else
+  {
+    // Invalid string was given
+    return GL_INVALID_ENUM;
+  }
+}
+
+
 /*
  * vim:et sw=2
  */
