@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2003 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2003-2004 The Boson Team (boson-devel@lists.sourceforge.net)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -30,7 +30,7 @@
 class BoApplication : public KApplication
 {
 public:
-	BoApplication(bool allowStyles = true);
+	BoApplication(bool allowStyles = true, bool enableGUI = true);
 	virtual ~BoApplication();
 };
 
