@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2002 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2002-2003 The Boson Team (boson-devel@lists.sourceforge.net)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -34,19 +34,24 @@
 #include "unit.h"
 #include "unitplugins.h"
 #include "bodebug.h"
+#include "boaction.h"
+#include "bosonlocalplayerinput.h"
+#include "bosonweapon.h"
 
 #include <klocale.h>
 
 BosonBigDisplayInput::BosonBigDisplayInput(BosonBigDisplayBase* parent) : BosonBigDisplayInputBase(parent)
 {
+ weaponId = -1;
 }
 
 BosonBigDisplayInput::~BosonBigDisplayInput()
 {
 }
 
-void BosonBigDisplayInput::actionClicked(const BoAction& action, QDataStream& stream, bool* send)
+void BosonBigDisplayInput::actionClicked(const BoMouseEvent& event)
 {
+ boDebug() << k_funcinfo << endl;
 // this method should not perform any tasks but rather send the input through
 // the KGameIO. this way it is very easy (it should be at least) to write a
 // computer player
@@ -56,70 +61,63 @@ void BosonBigDisplayInput::actionClicked(const BoAction& action, QDataStream& st
  if (selection()->isEmpty()) {
 	return;
  }
- if (!canvas()->onCanvas(action.canvasVector())) {
-	boError() << k_funcinfo << action.canvasVector().x() << "," << action.canvasVector().y() << " is not on the canvas!" << endl;
+ if (!canvas()->onCanvas(event.canvasVector())) {
+	boError() << k_funcinfo << event.canvasVector().x() << "," << event.canvasVector().y() << " is not on the canvas!" << endl;
 	return;
  }
 
  if (actionLocked()) {
+	bool unlock = false;
 	switch (actionType()) {
 		case ActionMove:
 		{
-			if (actionMoveWithoutAttack(stream, action.canvasVector())) {
-				*send = true;
-			}
+			unlock = actionMoveWithoutAttack(event.canvasVector());
 			break;
 		}
 		case ActionAttack:
 		{
-			Unit* unit = canvas()->findUnitAt(action.canvasVector());
+			Unit* unit = canvas()->findUnitAt(event.canvasVector());
 			if (unit) {
-				if (actionAttack(stream, action.canvasVector())) {
-					*send = true;
-				}
+				unlock = actionAttack(event.canvasVector());
 			} else {
-				actionMoveWithAttack(stream, action.canvasVector());
-				*send = true;
+				unlock = actionMoveWithAttack(event.canvasVector());
 			}
 			break;
 		}
+		case ActionDropBomb:
+		{
+			unlock = actionDropBomb(event.canvasVector());
+			break;
+		}
 		case ActionFollow:
 		{
-			Unit* unit = canvas()->findUnitAt(action.canvasVector());
+			Unit* unit = canvas()->findUnitAt(event.canvasVector());
 			if (unit) {
-				if (actionFollow(stream, action.canvasVector())) {
-					*send = true;
-				}
+				unlock = actionFollow(event.canvasVector());
 			}
 			break;
 		}
-		case ActionMine:
+		case ActionHarvest:
 		{
 			// TODO check if player clicked on oil/minerals
-			if (actionMine(stream, action.canvasVector())) {
-				*send = true;
-			}
+			unlock = actionHarvest(event.canvasVector());
 			break;
 		}
 		case ActionRepair:
 		{
-			if (actionRepair(stream, action.canvasVector())) {
-				*send = true;
-			}
+			unlock = actionRepair(event.canvasVector());
 			break;
 		}
-		case ActionBuild:
+		case ActionPlacementPreview:
 		{
-			if (actionBuild(stream, action.canvasVector())) {
-				*send = true;
-			}
+			unlock = actionBuild(event.canvasVector());
 			break;
 		}
 		default:
-			boError() << k_funcinfo << "Unknown actiontype for locked action: " << actionType() << endl;
+			boError() << k_funcinfo << "Unsupported actiontype for locked action: " << actionType() << endl;
 			break;
 	}
-	if (*send) {
+	if (unlock) {
 		// TODO: also play "cannot do that" sound
 		unlockAction();
 	}
@@ -127,8 +125,8 @@ void BosonBigDisplayInput::actionClicked(const BoAction& action, QDataStream& st
  }
 
  Unit* unit = 0;
- if (!localPlayer()->isFogged((int)(action.canvasVector().x() / BO_TILE_SIZE), (int)(action.canvasVector().y() / BO_TILE_SIZE))) {
-	unit = canvas()->findUnitAt(action.canvasVector());
+ if (!localPlayer()->isFogged((int)(event.canvasVector().x() / BO_TILE_SIZE), (int)(event.canvasVector().y() / BO_TILE_SIZE))) {
+	unit = canvas()->findUnitAt(event.canvasVector());
  }
  if (!unit) {
 	//FIXME: first check if a the unit can produce! even mobile units can
@@ -137,22 +135,24 @@ void BosonBigDisplayInput::actionClicked(const BoAction& action, QDataStream& st
 		if (selection()->count() == 1) {
 			// there are special things to do for a single selected unit
 			// (e.g. mining if the unit is a harvester)
-			if (actionMine(stream, action.canvasVector())) {
-				*send = true;
+			/*
+			 * AB: obsolete, as harvesting will take place on units
+			 * (->mines) only!
+			if (actionHarvest(event.canvasVector())) {
 				return;
 			}
+			*/
 		}
 
 		if (boConfig->RMBMovesWithAttack()) {
-			if (!actionMoveWithAttack(stream, action.canvasVector())) {
+			if (!actionMoveWithAttack(event.canvasVector())) {
 				return;
 			}
 		} else {
-			if (!actionMoveWithoutAttack(stream, action.canvasVector())) {
+			if (!actionMoveWithoutAttack(event.canvasVector())) {
 				return;
 			}
 		}
-		*send = true;
 	} else {
 		// TODO: another option: add the waypoint to the facility and
 		// apply it to any unit that gets constructed by that facility.
@@ -161,58 +161,62 @@ void BosonBigDisplayInput::actionClicked(const BoAction& action, QDataStream& st
 		// now ! (ActionBuild)
 	}
  } else { // there is a unit - attack it?
-	if ((localPlayer()->isEnemy(unit->owner()) || action.forceAttack()) &&
+	if ((localPlayer()->isEnemy(unit->owner()) || event.forceAttack()) &&
 			selection()->canShootAt(unit)) {
 		// attack the unit
-		if (!actionAttack(stream, action.canvasVector())) {
+		if (!actionAttack(event.canvasVector())) {
 			return;
 		}
-		*send = true;
 
 	} else if (localPlayer()->isEnemy(unit->owner())) {
 		// a non-friendly unit, but the selection cannot shoot
 		// we probably won't do anything here
 		// IDEA: what about "I cannot shoot that!" sound?
-	} else {
+	} else if (localPlayer()->isAllied(unit->owner())) {
 		// click on a friendly unit
 		if (unit->isFacility() && unit->plugin(UnitPlugin::Repair)) {
 			// some kind of repairyard - repair all units
 			// (not yet implemented) //FIXME
 			// note that currently the unit can go to every friendly
 			// player, even non-local players
-			if (!actionRepair(stream, action.canvasVector())) {
+			if (!actionRepair(event.canvasVector())) {
 				return;
 			}
-			*send = true;
 		} else if (unit->properties(PluginProperties::Refinery)) {
 			const RefineryProperties* prop = (RefineryProperties*)unit->properties(PluginProperties::Refinery);
 			if((prop->canRefineMinerals() && selection()->hasMineralHarvester()) ||
 					(prop->canRefineOil() && selection()->hasOilHarvester())) {
 				// go to the refinery
-				if (!actionRefine(stream, action.canvasVector())) {
+				if (!actionRefine(event.canvasVector())) {
 					return;
 				}
-				*send = true;
 			}
 		} else if (selection()->hasMobileUnit() && unit->isMobile()) {
 			// Follow
-			if (!actionFollow(stream, action.canvasVector())) {
+			if (!actionFollow(event.canvasVector())) {
 				return;
 			}
-			*send = true;
 		} else {
 			// selection and clicked unit both are friendly
 			// no repairyard and no refinery
 			// (at least no valid)
 			// add other possibilities here
 		}
+	} else if (localPlayer()->isNeutral(unit->owner())) {
+		// click on a neutral unit
+		// note: this is NOT a friendly unit, so we won' repair it or
+		// so. but we won't shoot at it either (by default).
+
+		if (actionHarvest(event.canvasVector())) {
+			return;
+		}
 	}
  }
 
 }
 
 
-bool BosonBigDisplayInput::actionMine(QDataStream& stream, const BoVector3& canvasVector)
+bool BosonBigDisplayInput::actionHarvest(const BoVector3& canvasVector)
 {
  if (!canvas()) {
 	BO_NULL_ERROR(canvas());
@@ -222,21 +226,45 @@ bool BosonBigDisplayInput::actionMine(QDataStream& stream, const BoVector3& canv
 	BO_NULL_ERROR(selection());
 	return false;
  }
- MobileUnit* u = (MobileUnit*)selection()->leader();
- HarvesterPlugin* h = (HarvesterPlugin*)u->plugin(UnitPlugin::Harvester);
- if (!h) {
+ if (!localPlayerInput()) {
+	BO_NULL_ERROR(localPlayerInput());
 	return false;
  }
- if (h->canMine((canvas())->cellAt(canvasVector.x(), canvasVector.y()))) {
-	stream << (Q_UINT32)BosonMessage::MoveMine;
-	stream << (Q_ULONG)u->id();
-	stream << QPoint((int)canvasVector.x(), (int)canvasVector.y());
-	return true;
+ Unit* resourceUnit = canvas()->findUnitAt(canvasVector);
+ if (!resourceUnit) {
+	return false;
  }
- return false;
+ ResourceMinePlugin* resource = (ResourceMinePlugin*)resourceUnit->plugin(UnitPlugin::ResourceMine);
+ if (!resource) {
+	// there is no mine at destination
+	return false;
+ }
+ boDebug() << k_funcinfo << "clicked on a resource mine" << endl;
+
+ bool taken = false;
+ QPtrList<Unit> units = selection()->allUnits();
+ QPtrListIterator<Unit> it(units);
+ while (it.current()) {
+	Unit* u = it.current();
+	++it;
+
+	HarvesterPlugin* h = (HarvesterPlugin*)u->plugin(UnitPlugin::Harvester);
+	if (!h) {
+		continue;
+	}
+	boDebug() << u->id() << " is a harvester" << endl;
+	if (resource->isUsableTo(h)) {
+		boDebug() << k_funcinfo << u->id() << " will harvest at " << resource->unit()->id() << "," << canvasVector.y() << " which is " << h->unit()->id() << endl;
+		localPlayerInput()->harvest(h, resource);
+		taken = true;
+	} else {
+		boDebug() << k_funcinfo << u->id() << " cannot mine at " << resourceUnit->id() << endl;
+	}
+ }
+ return taken;
 }
 
-bool BosonBigDisplayInput::actionMoveWithoutAttack(QDataStream& stream, const BoVector3& canvasVector)
+bool BosonBigDisplayInput::actionMoveWithoutAttack(const BoVector3& canvasVector)
 {
  if (!selection()) {
 	BO_NULL_ERROR(selection());
@@ -246,34 +274,20 @@ bool BosonBigDisplayInput::actionMoveWithoutAttack(QDataStream& stream, const Bo
 	BO_NULL_ERROR(localPlayer());
 	return false;
  }
+ if (!localPlayerInput()) {
+	BO_NULL_ERROR(localPlayerInput());
+	return false;
+ }
  // AB: note that only x and y are relevant from canvasVector !
  // z is ignored
- QPtrList<Unit> list = selection()->allUnits();
- QPtrListIterator<Unit> it(list);
- // tell the clients we want to move units:
- stream << (Q_UINT32)BosonMessage::MoveMove;
- // We want to move without attacking
- stream << (Q_UINT8)0;
- // tell them where to move to:
- stream << QPoint((int)canvasVector.x(), (int)canvasVector.y());
- // tell them how many units:
- stream << (Q_UINT32)list.count();
- Unit* unit = 0;
- while (it.current()) {
-	if (!unit) {
-		unit = it.current();
-	}
-	// tell them which unit to move:
-	stream << (Q_ULONG)it.current()->id(); // MUST BE UNIQUE!
-	++it;
- }
- if (unit->owner() == localPlayer()) {
-	unit->playSound(SoundOrderMove);
+ localPlayerInput()->moveWithoutAttack(selection()->allUnits(), (int)canvasVector.x(), (int)canvasVector.y());
+ if (selection()->leader()->owner() == localPlayer()) {
+	selection()->leader()->playSound(SoundOrderMove);
  }
  return true;
 }
 
-bool BosonBigDisplayInput::actionMoveWithAttack(QDataStream& stream, const BoVector3& canvasVector)
+bool BosonBigDisplayInput::actionMoveWithAttack(const BoVector3& canvasVector)
 {
  if (!localPlayer()) {
 	BO_NULL_ERROR(localPlayer());
@@ -283,39 +297,29 @@ bool BosonBigDisplayInput::actionMoveWithAttack(QDataStream& stream, const BoVec
 	BO_NULL_ERROR(selection());
 	return false;
  }
+ if (!localPlayerInput()) {
+	BO_NULL_ERROR(localPlayerInput());
+	return false;
+ }
  // AB: note that only x and y are relevant from canvasVector !
  // z is ignored
- QPtrList<Unit> list = selection()->allUnits();
- QPtrListIterator<Unit> it(list);
- // tell the clients we want to move units:
- stream << (Q_UINT32)BosonMessage::MoveMove;
- // We want to move with attacking
- stream << (Q_UINT8)1;
- // tell them where to move to:
- stream << QPoint((int)canvasVector.x(), (int)canvasVector.y());
- // tell them how many units:
- stream << (Q_UINT32)list.count();
- Unit* unit = 0;
- while (it.current()) {
-	if (!unit) {
-		unit = it.current();
-	}
-	// tell them which unit to move:
-	stream << (Q_ULONG)it.current()->id(); // MUST BE UNIQUE!
-	++it;
- }
- if (unit->owner() == localPlayer()) {
-	unit->playSound(SoundOrderAttack);
+ localPlayerInput()->moveWithAttack(selection()->allUnits(), (int)canvasVector.x(), (int)canvasVector.y());
+ if (selection()->leader()->owner() == localPlayer()) {
+	selection()->leader()->playSound(SoundOrderMove);
  }
  return true;
 }
 
-bool BosonBigDisplayInput::actionBuild(QDataStream& stream, const BoVector3& canvasVector)
+bool BosonBigDisplayInput::actionBuild(const BoVector3& canvasVector)
 {
  if (!localPlayer()) {
 	BO_NULL_ERROR(localPlayer());
 	return false;
  }
+ if (!localPlayerInput()) {
+	BO_NULL_ERROR(localPlayerInput());
+	return false;
+ }
  if (!selection()) {
 	BO_NULL_ERROR(selection());
 	return false;
@@ -328,6 +332,7 @@ bool BosonBigDisplayInput::actionBuild(QDataStream& stream, const BoVector3& can
  if (!factory) {
 	return false;
  }
+ // FIXME: lot of this code should probably be moved to BosonLocalPlayerInput
  ProductionPlugin* production = (ProductionPlugin*)(factory->plugin(UnitPlugin::Production));
  if (!production || !production->hasProduction() || production->completedProductionId() <= 0) {
 	return false;
@@ -345,21 +350,21 @@ bool BosonBigDisplayInput::actionBuild(QDataStream& stream, const BoVector3& can
  }
 
  // create the new unit
- stream << (Q_UINT32)BosonMessage::MoveBuild;
- stream << (Q_UINT32)production->completedProductionType();
- stream << (Q_ULONG)factory->id();
- stream << (Q_UINT32)factory->owner()->id();
- stream << (Q_INT32)(canvasVector.x() / BO_TILE_SIZE);
- stream << (Q_INT32)(canvasVector.y() / BO_TILE_SIZE);
+ localPlayerInput()->build(production->completedProductionType(), factory,
+		(int)(canvasVector.x() / BO_TILE_SIZE), (int)(canvasVector.y() / BO_TILE_SIZE));
  return true;
 }
 
-bool BosonBigDisplayInput::actionAttack(QDataStream& stream, const BoVector3& canvasVector)
+bool BosonBigDisplayInput::actionAttack(const BoVector3& canvasVector)
 {
  if (!localPlayer()) {
 	BO_NULL_ERROR(localPlayer());
 	return false;
  }
+ if (!localPlayerInput()) {
+	BO_NULL_ERROR(localPlayerInput());
+	return false;
+ }
  if (!selection()) {
 	BO_NULL_ERROR(selection());
 	return false;
@@ -369,19 +374,7 @@ bool BosonBigDisplayInput::actionAttack(QDataStream& stream, const BoVector3& ca
 	return false;
  }
  Unit* unit = canvas()->findUnitAt(canvasVector);
- QPtrList<Unit> list = selection()->allUnits();
- QPtrListIterator<Unit> it(list);
- // tell the clients we want to attack:
- stream << (Q_UINT32)BosonMessage::MoveAttack;
- // tell them which unit to attack:
- stream << (Q_ULONG)unit->id();
- // tell them how many units attack:
- stream << (Q_UINT32)list.count();
- while (it.current()) {
-	// tell them which unit is going to attack:
-	stream << (Q_ULONG)it.current()->id(); // MUST BE UNIQUE!
-	++it;
- }
+ localPlayerInput()->attack(selection()->allUnits(), unit);
  Unit* u = selection()->leader();
  if (u->owner() == localPlayer()) {
 	u->playSound(SoundOrderAttack);
@@ -389,7 +382,31 @@ bool BosonBigDisplayInput::actionAttack(QDataStream& stream, const BoVector3& ca
  return true;
 }
 
-bool BosonBigDisplayInput::actionRepair(QDataStream& stream, const BoVector3& canvasVector)
+bool BosonBigDisplayInput::actionDropBomb(const BoVector3& canvasVector)
+{
+ if (!localPlayer()) {
+	BO_NULL_ERROR(localPlayer());
+	return false;
+ }
+ if (!localPlayerInput()) {
+	BO_NULL_ERROR(localPlayerInput());
+	return false;
+ }
+ if (!selection()) {
+	BO_NULL_ERROR(selection());
+	return false;
+ }
+ // AB: note that only x and y are relevant from canvasVector !
+ // z is ignored
+ localPlayerInput()->dropBomb(selection()->leader(), weaponId, (int)canvasVector.x(), (int)canvasVector.y());
+ weaponId = -1;
+ if (selection()->leader()->owner() == localPlayer()) {
+	selection()->leader()->playSound(SoundOrderAttack);
+ }
+ return true;
+}
+
+bool BosonBigDisplayInput::actionRepair(const BoVector3& canvasVector)
 {
  if (!selection()) {
 	BO_NULL_ERROR(selection());
@@ -399,6 +416,10 @@ bool BosonBigDisplayInput::actionRepair(QDataStream& stream, const BoVector3& ca
 	BO_NULL_ERROR(canvas());
 	return false;
  }
+ if (!localPlayerInput()) {
+	BO_NULL_ERROR(localPlayerInput());
+	return false;
+ }
  Unit* unit = canvas()->findUnitAt(canvasVector);
  QPtrList<Unit> allUnits = selection()->allUnits();
  QPtrList<Unit> list;
@@ -410,28 +431,14 @@ bool BosonBigDisplayInput::actionRepair(QDataStream& stream, const BoVector3& ca
 	}
 	++it;
  }
- it = QPtrListIterator<Unit>(list);
- // tell the clients we want to repair:
- stream << (Q_UINT32)BosonMessage::MoveRepair;
- // the owner of the repairyard (can also be an allied
- // player - not localplayer only)
- stream << (Q_UINT32)unit->owner()->id();
- // tell them where to repair the units:
- stream << (Q_ULONG)unit->id();
- // tell them how many units to be repaired:
- stream << (Q_UINT32)list.count();
- while (it.current()) {
-	// tell them which unit is going be repaired:
-	stream << (Q_ULONG)it.current()->id();
-	++it;
- }
+ localPlayerInput()->repair(list, unit);
 // TODO:
 // Unit* u = selection()->leader();
 // u->playSound(SoundOrderRepair);
  return true;
 }
 
-bool BosonBigDisplayInput::actionRefine(QDataStream& stream, const BoVector3& canvasVector)
+bool BosonBigDisplayInput::actionRefine(const BoVector3& canvasVector)
 {
  if (!selection()) {
 	BO_NULL_ERROR(selection());
@@ -441,6 +448,10 @@ bool BosonBigDisplayInput::actionRefine(QDataStream& stream, const BoVector3& ca
 	BO_NULL_ERROR(canvas());
 	return false;
  }
+ if (!localPlayerInput()) {
+	BO_NULL_ERROR(localPlayerInput());
+	return false;
+ }
  Unit* unit = canvas()->findUnitAt(canvasVector);
  const RefineryProperties* prop = (RefineryProperties*)unit->properties(PluginProperties::Refinery);
  if (!prop) {
@@ -463,27 +474,14 @@ bool BosonBigDisplayInput::actionRefine(QDataStream& stream, const BoVector3& ca
 	boError() << k_lineinfo << "MoveRefine: empty list!!" << endl;
 	return false;
  }
- QPtrListIterator<Unit> it(list);
- stream << (Q_UINT32)BosonMessage::MoveRefine;
- // the owner of the refinery (can also be an allied
- // player - not localplayer only)
- stream << (Q_UINT32)unit->owner()->id();
- // destination:
- stream << (Q_ULONG)unit->id();
- // how many units go to the refinery
- stream << (Q_UINT32)list.count();
- while (it.current()) {
-	// tell them which unit goes there
-	stream << (Q_ULONG)it.current()->id();
-	++it;
- }
+ localPlayerInput()->refine(list, unit);
 // TODO:
 // Unit* u = selection()->leader();
 // u->playSound(SoundOrderRefine);
  return true;
 }
 
-bool BosonBigDisplayInput::actionFollow(QDataStream& stream, const BoVector3& canvasVector)
+bool BosonBigDisplayInput::actionFollow(const BoVector3& canvasVector)
 {
  if (!localPlayer()) {
 	BO_NULL_ERROR(localPlayer());
@@ -497,20 +495,12 @@ bool BosonBigDisplayInput::actionFollow(QDataStream& stream, const BoVector3& ca
 	BO_NULL_ERROR(canvas());
 	return false;
  }
- Unit* unit = canvas()->findUnitAt(canvasVector);
- QPtrList<Unit> list = selection()->allUnits();
- QPtrListIterator<Unit> it(list);
- // tell the clients we want to follow:
- stream << (Q_UINT32)BosonMessage::MoveFollow;
- // tell them which unit to follow:
- stream << (Q_ULONG)unit->id();
- // tell them how many units follow:
- stream << (Q_UINT32)list.count();
- while (it.current()) {
-	// tell them which unit is going to follow:
-	stream << (Q_ULONG)it.current()->id(); // MUST BE UNIQUE!
-	++it;
+ if (!localPlayerInput()) {
+	BO_NULL_ERROR(localPlayerInput());
+	return false;
  }
+ Unit* unit = canvas()->findUnitAt(canvasVector);
+ localPlayerInput()->follow(selection()->allUnits(), unit);
  Unit* u = selection()->leader();
  if (u->owner() == localPlayer()) {
 	u->playSound(SoundOrderMove);
@@ -524,7 +514,7 @@ void BosonBigDisplayInput::updatePlacementPreviewData()
  BO_CHECK_NULL_RET(selection());
  BO_CHECK_NULL_RET(canvas());
  bigDisplay()->setPlacementPreviewData(0, false);
- if (!actionLocked() || actionType() != ActionBuild) {
+ if (!actionLocked() || actionType() != ActionPlacementPreview) {
 	return;
  }
  if (!selection() || selection()->isEmpty() || !selection()->leader()) {
@@ -553,56 +543,33 @@ void BosonBigDisplayInput::updatePlacementPreviewData()
 
 }
 
-void BosonBigDisplayInput::unitAction(int actionType)
+void BosonBigDisplayInput::action(const BoSpecificAction& action)
 {
+ boDebug() << k_funcinfo << endl;
  BO_CHECK_NULL_RET(localPlayer());
  BO_CHECK_NULL_RET(selection());
- switch ((UnitAction)actionType) {
+
+ switch (action.type()) {
 	case ActionFollow:
-	case ActionMine:
+	case ActionHarvest:
 	case ActionRepair:
 	case ActionMove:
 		setCursorType(CursorMove);
 		break;
-	case ActionBuild:
+	case ActionPlacementPreview:
 		break;
 	case ActionAttack:
 		setCursorType(CursorAttack);
 		break;
-	case ActionStop:
-	{
-		if (selection()->isEmpty()) {
-			boError() << k_funcinfo << "Selection is empty!" << endl;
-			return;
-		}
-		// Stop all selected units
-		// I REALLY hope I'm doing this correctly
-		// TODO: should be in actionStop()
-		QPtrList<Unit> list = selection()->allUnits();
-		QPtrListIterator<Unit> it(list);
-		QByteArray b;
-		QDataStream stream(b, IO_WriteOnly);
-
-		// tell the clients we want to move units:
-		stream << (Q_UINT32)BosonMessage::MoveStop;
-		// tell them how many units:
-		stream << (Q_UINT32)list.count();
-		while (it.current()) {
-			// tell them which unit to move:
-			stream << (Q_ULONG)it.current()->id(); // MUST BE UNIQUE!
-			++it;
-		}
-
-		QDataStream msg(b, IO_WriteOnly);
-		localPlayer()->forwardInput(msg);
-		unlockAction();
-		return;
-	}
+	case ActionDropBomb:
+		weaponId = action.weapon()->id();
+		setCursorType(CursorAttack);
+		break;
 	default:
-		boError() << k_funcinfo << "Unknown actionType: " << actionType << endl;
+		boError() << k_funcinfo << "Invalid or unsupported action! type: " << action.type() << endl;
 		return;
  }
- setActionType((UnitAction)actionType);
+ setActionType(action.type());
  lockAction();
 }
 
@@ -722,17 +689,10 @@ void BosonBigDisplayInput::slotMoveSelection(int cellX, int cellY)
  if (selection()->isEmpty()) {
 	return;
  }
- QByteArray buffer;
- QDataStream stream(buffer, IO_WriteOnly);
- bool send = false;
- BoAction action;
- action.setCanvasVector(BoVector3((float)(cellX * BO_TILE_SIZE + BO_TILE_SIZE / 2),
+ BoMouseEvent event;
+ event.setCanvasVector(BoVector3((float)(cellX * BO_TILE_SIZE + BO_TILE_SIZE / 2),
 		(float)(cellY * BO_TILE_SIZE + BO_TILE_SIZE / 2),
 		0.0f));
- actionClicked(action, stream, &send);
- if (send) {
-	QDataStream msg(buffer, IO_ReadOnly);
-	localPlayer()->forwardInput(msg, true);
- }
+ actionClicked(event);
 }
 
