@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2002-2003 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2002-2004 The Boson Team (boson-devel@lists.sourceforge.net)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -107,7 +107,7 @@ public:
 	 * Use @ref loadPlayField to load <em>all</em> data. preLoadPlayField is
 	 * much faster than @ref loadPlayField.
 	 **/
-	bool preLoadPlayField(const QString& identifier);
+	bool preLoadPlayField(const QString& file);
 
 	/**
 	 * Load the playfield from @p files to this object.
@@ -130,8 +130,6 @@ public:
 	 **/
 	QByteArray exportTexMap(unsigned int texture) const;
 
-	bool savePlayField(const QString& fileName);
-
 	/**
 	 * Make @p map the new map of this playfield. Called by the editor
 	 * only.
@@ -214,12 +212,6 @@ public:
 	 **/
 	static QString defaultPlayField();
 
-signals:
-	/**
-	 * Emitted when the map changes. Note that this can even be 0!
-	 **/
-	void signalNewMap(BosonMap*);
-
 protected:
 	bool loadDescriptionFromFile(const QByteArray& xml);
 	bool loadMapFromFile(const QByteArray& xml, const QByteArray& heightMapImage, const QByteArray& texMap);
@@ -228,6 +220,29 @@ protected:
 	QByteArray saveMapToFile();
 	QByteArray saveTexMapToFile();
 
+	/**
+	 * This takes a set of files, as loaded by @ref loadFromDiskToFiles, and
+	 * checks whether they have the most current file format. If they don't
+	 * have, this method tries to convert them.
+	 *
+	 * @since Boson 0.9.1
+	 **/
+	bool convertFilesToCurrentFormat(QMap<QString, QByteArray>& destFiles);
+
+	/**
+	 * Called by the above method. This one should care about the actual
+	 * conversion.
+	 *
+	 * Here you are meant to place the actual conversion code.
+	 * @param set to TRUE when the @p version has been recognized and (!)
+	 * used to convert the files to a new format (i.e. at least changed
+	 * the version number). Otherwise set to FALSE (i.e. if the files have
+	 * not been touched at all).
+	 * @return FALSE, if the file should have been converted, but we failed
+	 * while doing so. Otherwise TRUE.
+	 **/
+	bool convertFilesToCurrentFormat(QMap<QString, QByteArray>& destFiles, unsigned int version, bool* handled);
+
 	/**
 	 * @return A list of all playfields (.bpf files) that are found on this
 	 * computer. This does not check whether they got already loaded.
