@@ -198,6 +198,10 @@ bool EditorBigDisplayInput::actionPlace(const BoVector3& canvasVector)
 	BO_NULL_ERROR(localPlayer())
 	return false;
  }
+ if (!localPlayerInput()) {
+	BO_NULL_ERROR(localPlayerInput());
+	return false;
+ }
  bool ret = false;
  int x = (int)(canvasVector.x()) / BO_TILE_SIZE;
  int y = (int)(canvasVector.y()) / BO_TILE_SIZE;
@@ -310,6 +314,10 @@ bool EditorBigDisplayInput::actionPlace(const BoVector3& canvasVector)
 bool EditorBigDisplayInput::actionChangeHeight(const BoVector3& canvasVector, bool up)
 {
  boDebug() << k_funcinfo << endl;
+ if (!localPlayerInput()) {
+	BO_NULL_ERROR(localPlayerInput());
+	return false;
+ }
  if (!canvas()) {
 	BO_NULL_ERROR(canvas());
 	return false;
@@ -325,13 +333,13 @@ bool EditorBigDisplayInput::actionChangeHeight(const BoVector3& canvasVector, bo
  int cornerX = 0;
  int cornerY = 0;
  // we need the corner that was clicked, not the cell!
- if (((int)canvasVector.x()) % BO_TILE_SIZE >= BO_TILE_SIZE) {
+ if (((int)canvasVector.x()) % BO_TILE_SIZE >= BO_TILE_SIZE / 2) {
 	// a right corner
 	cornerX = cellX + 1;
  } else {
 	cornerX = cellX;
  }
- if (((int)canvasVector.x()) % BO_TILE_SIZE >= BO_TILE_SIZE) {
+ if (((int)canvasVector.y()) % BO_TILE_SIZE >= BO_TILE_SIZE / 2) {
 	cornerY = cellY + 1;
  } else {
 	cornerY = cellY;
@@ -379,17 +387,26 @@ void EditorBigDisplayInput::deleteSelectedUnits()
 {
  BO_CHECK_NULL_RET(selection());
  BO_CHECK_NULL_RET(canvas());
+ boDebug() << k_funcinfo << endl;
  if (selection()->isEmpty()) {
 	boDebug() << k_funcinfo << "no unit selected" << endl;
 	return;
  }
  QPtrList<Unit> units = selection()->allUnits();
  selection()->clear();
+ QByteArray b;
+ QDataStream s(b, IO_WriteOnly);
+ s << (Q_UINT32)BosonMessage::MoveEditor;
+ s << (Q_UINT32)BosonMessage::MoveDeleteItems;
+ s << (Q_UINT32)units.count();
  QPtrListIterator<Unit> it(units);
+ QPtrList<BosonItem> items;
  for (; it.current(); ++it) {
-	canvas()->removeUnit(it.current());
+	s << (Q_ULONG)it.current()->id();
  }
- canvas()->deleteUnits(&units);
+
+ QDataStream msg(b, IO_ReadOnly);
+ localPlayerInput()->sendInput(msg);
 }
 
 void EditorBigDisplayInput::updatePlacementPreviewData()
