@@ -27,6 +27,7 @@ class BosonConfig;
 class OptionsWidget;
 class Player;
 class Boson;
+class BoFontInfo;
 
 /**
  * @author Andreas Beckermann <b_mann@gmx.de>
@@ -46,6 +47,8 @@ public:
 signals:
 	void signalCursorChanged(int index, const QString& cursorDir);
 	void signalCmdBackgroundChanged(const QString& file);
+	void signalOpenGLSettingsUpdated();
+	void signalFontChanged(const BoFontInfo&);
 
 	/**
 	 * Emitted when apply or ok is clicked. At this point all options
