@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 1999-2000,2001-2003 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 1999-2000,2001-2004 The Boson Team (boson-devel@lists.sourceforge.net)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -35,9 +35,6 @@
 #include <ksimpleconfig.h>
 #include <klocale.h>
 
-#include <qpixmap.h>
-#include <qimage.h>
-#include <qbitmap.h>
 #include <qintdict.h>
 #include <qdir.h>
 
@@ -55,7 +52,7 @@ public:
 };
 
 static int defaultColorIndex = 0;
-QRgb default_color[BOSON_MAX_PLAYERS] = {
+QRgb default_color[BOSON_MAX_PLAYERS + 1] = { // AB: + 1 because of neutral player
 	qRgb(0,0,255),
 	qRgb(0,255,0),
 	qRgb(255,0,0),
@@ -66,6 +63,7 @@ QRgb default_color[BOSON_MAX_PLAYERS] = {
 	qRgb(255,0,127),
 	qRgb(0,127,255),
 	qRgb(0,127,127),
+	qRgb(127,0,255),
 };
 
 SpeciesTheme::SpeciesTheme(const QString& speciesDir, const QColor& teamColor)
@@ -190,7 +188,7 @@ QPixmap* SpeciesTheme::pixmap(const QString& name)
  return mData->pixmap(name);
 }
 
-BoAction* SpeciesTheme::action(const QString& name)
+const BoAction* SpeciesTheme::action(const QString& name) const
 {
  return mData->action(name);
 }
@@ -213,11 +211,16 @@ bool SpeciesTheme::loadTechnologies()
  for(it = techs.begin(); it != techs.end(); ++it) {
 	boDebug() << k_funcinfo << "Loading upgrade from group " << *it << endl;
 	UpgradeProperties* tech = new UpgradeProperties(this);
-	tech->load(&cfg, *it);
+	if (!tech->load(&cfg, *it)) {
+		boError() << k_funcinfo << *it << " could not be loaded" << endl;
+		delete tech;
+		continue;
+	}
 	if (!d->mTechnologies.find(tech->id())) {
 		d->mTechnologies.insert(tech->id(), tech);
 	} else {
 		boError() << k_funcinfo << "Technology with id " << tech->id() << " already there!" << endl;
+		delete tech;
 	}
  }
  return true;
@@ -229,9 +232,9 @@ void SpeciesTheme::loadObjects()
  mData->loadObjects(teamColor());
 }
 
-QString SpeciesTheme::unitModelFile()
+QStringList SpeciesTheme::unitModelFiles()
 {
- return SpeciesData::unitModelFile();
+ return SpeciesData::unitModelFiles();
 }
 
 BosonModel* SpeciesTheme::unitModel(unsigned long int unitType)
@@ -618,19 +621,19 @@ void SpeciesTheme::loadGeneralSounds()
  sound()->addSounds(themePath(), sounds);
 }
 
-QIntDict<UpgradeProperties> SpeciesTheme::technologyList() const
+const QIntDict<UpgradeProperties>& SpeciesTheme::technologyList() const
 {
  return d->mTechnologies;
 }
 
-void SpeciesTheme::loadParticleSystems()
+void SpeciesTheme::loadEffects()
 {
- mData->loadParticleSystemProperties();
+ mData->loadEffectProperties();
 }
 
-const BosonParticleSystemProperties* SpeciesTheme::particleSystemProperties(unsigned long int id)
+const BosonEffectProperties* SpeciesTheme::effectProperties(unsigned long int id)
 {
- return mData->particleSystemProperties(id);
+ return mData->effectProperties(id);
 }
 
 BosonModel* SpeciesTheme::objectModel(const QString& name) const
