@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 1999-2000,2001-2003 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 1999-2000,2001-2004 The Boson Team (boson-devel@lists.sourceforge.net)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -41,10 +41,15 @@
 #include "sound/bosonaudiointerface.h"
 #include "kgameunitdebug.h"
 #include "kgameplayerdebug.h"
-#include "kgamecelldebug.h"
 #include "bosonprofilingdialog.h"
 #include "bosondata.h"
 #include "bosongroundtheme.h"
+#include "bofullscreen.h"
+#include "bosonlocalplayerinput.h"
+//#include "kgamecelldebug.h"
+
+#include <kgamedebugdialog.h>
+#include <kgameio.h>
 
 #include <kapplication.h>
 #include <klocale.h>
@@ -59,7 +64,6 @@
 #include <kstandarddirs.h>
 #include <kfiledialog.h>
 #include <kglobal.h>
-#include <kgame/kgamedebugdialog.h>
 
 #include <qcursor.h>
 #include <qwidgetstack.h>
@@ -119,7 +123,6 @@ public:
 TopWidget::TopWidget() : KDockMainWindow(0, "topwindow")
 {
  d = new TopWidgetPrivate;
- mPlayField = 0;
 #if KDE_VERSION < 310
  d->mLoadingDockConfig = false;
 #endif
@@ -129,20 +132,9 @@ TopWidget::TopWidget() : KDockMainWindow(0, "topwindow")
  mMainDock->setEnableDocking(KDockWidget::DockNone);
  initGameDockWidgets(false);
 
- if (!BosonGroundTheme::createGroundThemeList()) {
-	boError() << k_funcinfo << "Unable to create groundTheme list" << endl;
-	KMessageBox::sorry(this, i18n("Unable to load groundThemes. Check your installation!"));
-	exit(1);
- }
- if (!BosonPlayField::preLoadAllPlayFields()) {
-	boError() << k_funcinfo << "Unable to preload playFields" << endl;
-	KMessageBox::sorry(this, i18n("Unable to preload playFields. Check your installation!"));
-	exit(1);
- }
  boAudio->setSound(boConfig->sound());
  boAudio->setMusic(boConfig->music());
 
-
  d->mStartup = new BosonStartupWidget(mMainDock);
  connect(d->mStartup, SIGNAL(signalAddLocalPlayer()),
 		this, SLOT(slotAddLocalPlayer()));
@@ -166,11 +158,6 @@ TopWidget::TopWidget() : KDockMainWindow(0, "topwindow")
  initKActions();
  initStatusBar();
 
- // this will also call slotResetGame() and therefore init the game
- d->mStartup->slotShowWelcomeWidget();
-
- loadInitialDockConfig();
-
  d->mIface = new BoDebugDCOPIface();
 
  initDisplayManager();
@@ -192,9 +179,32 @@ TopWidget::~TopWidget()
  boDebug() << k_funcinfo << "done" << endl;
 }
 
+QString TopWidget::checkInstallation()
+{
+ QPixmap p1(locate("data", "boson/pics/boson-startup-bg.png"));
+ QPixmap p2(locate("data", "boson/pics/boson-startup-logo.png"));
+  if (p1.isNull() || p2.isNull()) {
+	return i18n("You seem not to have Boson data files installed!\n Please install data package of Boson and restart Boson.");
+}
+ if (!BosonGroundTheme::createGroundThemeList()) {
+	return i18n("Unable to load groundThemes. Check your installation!");
+ }
+ if (!BosonPlayField::preLoadAllPlayFields()) {
+	return i18n("Unable to preload playFields. Check your installation!");
+ }
+ return QString::null;
+}
+
 void TopWidget::initDisplayManager()
 {
  d->mDisplayManager = new BoDisplayManager(0);
+
+ // add an initial display. this should happen asap, as we need the OpenGL
+ // context for every texture that is to be loaded.
+ // the display will not be deleted before the program is quit, so that we don't
+ // have to load the textures several times.
+ d->mDisplayManager->addInitialDisplay();
+
  d->mDisplayManager->hide();
 }
 
@@ -275,19 +285,6 @@ void TopWidget::slotToggleCmdFrameDockVisible()
  slotCheckGameDockStatus();
 }
 
-void TopWidget::slotRemoveFromGUIFactory(QObject* obj)
-{
- if (!obj) {
-	return;
- }
- if (obj->inherits("BosonWidgetBase")) {
-	BosonWidgetBase* w = (BosonWidgetBase*)obj;
-	if (factory()) {
-		factory()->removeClient(w);
-	}
- }
-}
-
 void TopWidget::saveProperties(KConfig *config)
 {
  // the 'config' object points to the session managed
@@ -315,7 +312,7 @@ void TopWidget::initKActions()
  // Main actions: Game start/end and quit
 
  //FIXME: slotNewGame() is broken - endGame() is enough for now.
-// (void)KStdGameAction::gameNew(this, SLOT(slotNewGame()), actionCollection()); 
+// (void)KStdGameAction::gameNew(this, SLOT(slotNewGame()), actionCollection());
 // (void)KStdGameAction::quit(this, SLOT(close()), actionCollection());
 
  // Settings
@@ -372,24 +369,6 @@ void TopWidget::initBoson()
 		this, SLOT(slotEditorNewMap(const QByteArray&)));
 }
 
-void TopWidget::initPlayer()
-{
- boWarning() << k_funcinfo << endl;
- Player* p = new Player;
- slotChangeLocalPlayer(p);
-}
-
-void TopWidget::initPlayField()
-{
- delete mPlayField;
- mPlayField = new BosonPlayField(this);
-}
-
-void TopWidget::initCanvas()
-{
- boGame->createCanvas();
-}
-
 void TopWidget::initStatusBar()
 {
  KStatusBar* bar = statusBar();
@@ -412,9 +391,9 @@ void TopWidget::initStatusBar()
  bar->addWidget(resources);
 
  QHBox* debug = new QHBox(bar);
- (void)new QLabel(i18n("Particles: "), debug);
- QLabel* particlesLabel = new QLabel(QString::number(0), debug);
- connect(this, SIGNAL(signalParticlesCountUpdated(int)), particlesLabel, SLOT(setNum(int)));
+ (void)new QLabel(i18n("Effects: "), debug);
+ QLabel* effectsLabel = new QLabel(QString::number(0), debug);
+ connect(this, SIGNAL(signalEffectsCountUpdated(int)), effectsLabel, SLOT(setNum(int)));
  (void)new QLabel(i18n("Canvas items: "), debug);
  QLabel* itemsLabel = new QLabel(QString::number(0), debug);
  connect(this, SIGNAL(signalCanvasItemsCountUpdated(int)), itemsLabel, SLOT(setNum(int)));
@@ -447,6 +426,7 @@ void TopWidget::initBosonWidget()
 	return;
  }
  BO_CHECK_NULL_RET(d->mDisplayManager);
+ BO_CHECK_NULL_RET(boGame);
  if (boGame->gameMode()) {
 	BosonWidget* w = new BosonWidget(mMainDock);
 	connect(w, SIGNAL(signalSaveGame()), this, SLOT(slotSaveGame()));
@@ -458,38 +438,36 @@ void TopWidget::initBosonWidget()
 	d->mBosonWidget = w;
  }
 
- connect(d->mBosonWidget, SIGNAL(destroyed(QObject*)), this, SLOT(slotRemoveFromGUIFactory(QObject*)));
- connect(d->mBosonWidget, SIGNAL(signalLoadBosonGameDock()), this, SLOT(slotLoadBosonGameDock()));
- connect(d->mBosonWidget, SIGNAL(signalToggleChatVisible()), this, SLOT(slotToggleChatDockVisible()));
- connect(d->mBosonWidget, SIGNAL(signalToggleCmdFrameVisible()), this, SLOT(slotToggleCmdFrameDockVisible()));
- connect(d->mBosonWidget, SIGNAL(signalCheckDockStatus()), this, SLOT(slotCheckGameDockStatus()));
- connect(d->mBosonWidget, SIGNAL(signalChangeLocalPlayer(Player*)), this, SLOT(slotChangeLocalPlayer(Player*)));
- connect(d->mBosonWidget, SIGNAL(signalEndGame()), this, SLOT(slotEndGame()));
- connect(d->mBosonWidget, SIGNAL(signalQuit()), this, SLOT(close()));
- connect(d->mBosonWidget, SIGNAL(signalMobilesCount(int)), this, SIGNAL(signalSetMobilesCount(int)));
- connect(d->mBosonWidget, SIGNAL(signalFacilitiesCount(int)), this, SIGNAL(signalSetFacilitiesCount(int)));
- connect(d->mBosonWidget, SIGNAL(signalOilUpdated(int)), this, SIGNAL(signalOilUpdated(int)));
- connect(d->mBosonWidget, SIGNAL(signalMineralsUpdated(int)), this, SIGNAL(signalMineralsUpdated(int)));
+ connect(d->mBosonWidget, SIGNAL(signalLoadBosonGameDock()),
+		this, SLOT(slotLoadBosonGameDock()));
+ connect(d->mBosonWidget, SIGNAL(signalToggleChatVisible()),
+		this, SLOT(slotToggleChatDockVisible()));
+ connect(d->mBosonWidget, SIGNAL(signalToggleCmdFrameVisible()),
+		this, SLOT(slotToggleCmdFrameDockVisible()));
+ connect(d->mBosonWidget, SIGNAL(signalCheckDockStatus()),
+		this, SLOT(slotCheckGameDockStatus()));
+ connect(d->mBosonWidget, SIGNAL(signalChangeLocalPlayer(Player*)),
+		this, SLOT(slotChangeLocalPlayer(Player*)));
+ connect(d->mBosonWidget, SIGNAL(signalEndGame()),
+		this, SLOT(slotEndGame()));
+ connect(d->mBosonWidget, SIGNAL(signalQuit()),
+		this, SLOT(close()));
+ connect(d->mBosonWidget, SIGNAL(signalMobilesCount(int)),
+		this, SIGNAL(signalSetMobilesCount(int)));
+ connect(d->mBosonWidget, SIGNAL(signalFacilitiesCount(int)),
+		this, SIGNAL(signalSetFacilitiesCount(int)));
+ connect(d->mBosonWidget, SIGNAL(signalOilUpdated(int)),
+		this, SIGNAL(signalOilUpdated(int)));
+ connect(d->mBosonWidget, SIGNAL(signalMineralsUpdated(int)),
+		this, SIGNAL(signalMineralsUpdated(int)));
 
- initGameDockWidgets(false);
  d->mBosonWidget->setDisplayManager(d->mDisplayManager);
 
- // at this point the startup widget should already have called
- // slotChangeLocalPlayer()! if not .. then we're in trouble here...
- if (!boGame->localPlayer()) {
-	boWarning() << k_funcinfo << "NULL local player - might cause trouble here!" << endl;
-	// do not return, since it might even be allowed for editor (currently
-	// it is not)
- }
- changeLocalPlayer(boGame->localPlayer(), false);
  d->mBosonWidget->init(d->mChatDock, d->mCommandFrameDock); // this depends on several virtual methods and therefore can't be called in the c'tor
 
  factory()->addClient(d->mBosonWidget); // XMLClient-stuff. needs to be called *after* creation of KAction objects, so outside BosonWidget might be a good idea :-)
 // createGUI("bosonui.rc", false);
 
- // finally add the initial display
- // note that this call also loads the cursor
- d->mBosonWidget->addInitialDisplay();
 }
 
 void TopWidget::slotPlayFieldChanged(const QString& id)
@@ -498,7 +476,6 @@ void TopWidget::slotPlayFieldChanged(const QString& id)
 	boError() << k_funcinfo << "NULL starting object!!" << endl;
 	return;
  }
- d->mStarting->setPlayFieldId(id);
 }
 
 void TopWidget::slotStartNewGame()
@@ -512,25 +489,34 @@ void TopWidget::slotStartNewGame()
 	boError() << k_funcinfo << "NULL startup widget" << endl;
 	return;
  }
- boDebug() << k_funcinfo << endl;
+ boDebug(270) << k_funcinfo << endl;
+
+ // AB: we need to have boGame->localPlayer() be valid and non-NULL for the
+ // starup widgets (atm).
+ // but for the actual starting process it is much cleaner, if the local player
+ // is NULL and gets applied at the end only.
+ slotChangeLocalPlayer(0);
 
  // Save initial dock config
  saveInitialDockConfig();
 
  d->mStartup->showLoadingWidget();
 
- d->mStarting->setDestPlayField(mPlayField);
+ initGameDockWidgets(false); // dock the widgets to their default location and hide them
 
- initCanvas();
  initBosonWidget();
 
- changeLocalPlayer(boGame->localPlayer());
- d->mBosonWidget->initPlayer();
+ if (d->mBosonWidget->localPlayer()) {
+	boWarning(270) << k_funcinfo << "localPlayer should be NULL until game is started!" << endl;
+ }
 
  // this will take care of all data loading, like models, textures and so. this
  // also initializes the map and will send IdStartScenario - in short this will
  // start the game. Once it's done it'll send IdGameIsStarted (see
  // Boson::signalGameStarted())
+ //
+ // Note that this return immediately - the loading is started using a
+ // QTimer::singleShot()
  d->mStarting->startNewGame();
 }
 
@@ -550,13 +536,19 @@ void TopWidget::slotLoadGame(const QString& fileName)
  }
 
  // load the file into memory
- if (!d->mStarting->loadGame(fileName)) {
+ QByteArray data = d->mStarting->loadGame(fileName);
+ if (data.size() == 0) {
 	boError() << k_funcinfo << "failed loading from file" << endl;
 	return;
  }
 
+ QByteArray buffer;
+ QDataStream stream(buffer, IO_WriteOnly);
+ stream << (Q_INT8)1; // game mode
+ stream << data;
+
  // actually start the game
- QTimer::singleShot(0, this, SLOT(slotStartNewGame()));
+ boGame->sendMessage(buffer, BosonMessage::IdNewGame);
 }
 
 void TopWidget::slotSaveGame(const QString& fileName, const QString& description)
@@ -614,16 +606,6 @@ void TopWidget::slotCancelLoadSave()
  }
 }
 
-void TopWidget::slotAssignMap()
-{
- if (boGame->gameStatus() != KGame::Init) {
-	boWarning() << k_funcinfo << "not in Init status" << endl;
-	return;
- }
- boDebug() << k_funcinfo << endl;
- d->mBosonWidget->initMap();
-}
-
 void TopWidget::slotToggleSound()
 {
  boAudio->setSound(!boAudio->sound());
@@ -651,9 +633,9 @@ void TopWidget::slotConfigureKeys()
 void TopWidget::slotToggleFullScreen()
 {
  if (d->mActionFullScreen->isChecked()) {
-	showFullScreen();
+	BoFullScreen::enterMode(-1);
  } else {
-	showNormal();
+	BoFullScreen::leaveFullScreen();
  }
 }
 
@@ -668,7 +650,6 @@ void TopWidget::endGame()
  }
  if (d->mBosonWidget) {
 	d->mBosonWidget->quitGame();
-//	disconnect(d->mBosonWidget, 0, 0, 0); // we must not do this, so that slotRemoveFromGUIFactory() is called. do we need this disconnect at all?
 	d->mStatusBarTimer.stop();
 	// This prevent wrong dock config from getting saved when loading fails
 	if (boGame->gameStatus() != KGame::Init) {
@@ -678,9 +659,9 @@ void TopWidget::endGame()
  delete d->mBosonWidget;
  d->mBosonWidget = 0;
  Boson::deleteBoson();  // Easiest way to reset game info
- delete mPlayField;
- mPlayField = 0;
  hideGameDockWidgets(true);
+ delete d->mStarting;
+ d->mStarting = 0;
  boDebug() << k_funcinfo << "done" << endl;
 }
 
@@ -690,7 +671,6 @@ void TopWidget::reinitGame()
 
  delete d->mStarting;
  d->mStarting = new BosonStarting(this); // manages startup of games
- connect(d->mStarting, SIGNAL(signalAssignMap()), this, SLOT(slotAssignMap()));
  connect(d->mStarting, SIGNAL(signalLoadingReset()),
 		d->mStartup, SLOT(slotLoadingReset()));
  connect(d->mStarting, SIGNAL(signalLoadingSetAdmin(bool)),
@@ -713,7 +693,6 @@ void TopWidget::reinitGame()
 		this, SLOT(slotStartingFailed()));
 
  initBoson();
- initPlayField();
 
  d->mActionStatusbar->setChecked(false);
  slotToggleStatusbar();
@@ -819,7 +798,7 @@ void TopWidget::loadInitialDockConfig()
  conf->setGroup("BosonInitialDock");
  bool fs = conf->readBoolEntry("FullScreen", false);
  d->mActionFullScreen->setChecked(fs);
- 
+
  if (isVisible()) {
 	slotToggleFullScreen();
 	readDockConfig(kapp->config(), "BosonInitialDock");
@@ -919,7 +898,7 @@ void TopWidget::slotUpdateStatusBar()
  BO_CHECK_NULL_RET(stat);
  // AB: some statusbar labels are *not* updated here (e.g. minerals and units),
  // but whenever their value actually changes.
- emit signalParticlesCountUpdated(stat->particleSystemsCount());
+ emit signalEffectsCountUpdated(stat->effectsCount());
  emit signalCanvasItemsCountUpdated(stat->allItemsCount());
  emit signalCanvasAnimationsCountUpdated(stat->animationsCount());
 
@@ -968,31 +947,36 @@ void TopWidget::showHideMenubar()
 
 
 
-void TopWidget::changeLocalPlayer(Player* p, bool init)
+void TopWidget::changeLocalPlayer(Player* p)
 {
  boDebug() << k_funcinfo << p << endl;
 
  // AB: note that both, p == 0 AND p == currentplayer are valid and must be executed!
- if (d->mStarting) {
-	d->mStarting->setLocalPlayer(p);
- }
  if (!boGame) {
 	boError() << k_funcinfo << "NULL game object" << endl;
 	return;
  }
- // AB: d->mBosonWidget->setLocalPlayer() also calls Boson::setLocalPlayer(),
- // but it is NULL when the startup widgets call it
- boGame->setLocalPlayer(p);
  if (d->mBosonWidget) {
-	d->mBosonWidget->setLocalPlayer(p, init);
+	d->mBosonWidget->setLocalPlayer(p);
  }
+
+ // AB: note: the startup widgets don't need to know the new local player
+ // -> they need the unique local player only (new game widget), which is set
+ // right after construction of the player.
 }
 
 void TopWidget::slotEndGame()
 {
- int answer = KMessageBox::warningYesNo(this, i18n("Are you sure you want to end this game?"),
+ BO_CHECK_NULL_RET(boGame);
+ int answer;
+ if (boGame->gameMode()) {
+	answer = KMessageBox::warningYesNo(this, i18n("Are you sure you want to end this game?"),
 		i18n("Are you sure?"), KStdGuiItem::yes(), KStdGuiItem::no(), "ConfirmEndGame");
- if (answer == KMessageBox::No) {
+ } else {
+	answer = KMessageBox::warningYesNo(this, i18n("Are you sure you want to end this editor session?"),
+		i18n("Are you sure?"), KStdGuiItem::yes(), KStdGuiItem::no(), "ConfirmEndGame");
+ }
+ if (answer != KMessageBox::Yes) {
 	return;
  }
  slotGameOver();
@@ -1004,39 +988,101 @@ void TopWidget::slotAddLocalPlayer()
 	boError() << k_funcinfo << "NULL game object" << endl;
 	return;
  }
- if (boGame->localPlayer()) {
-	boError() << k_funcinfo << "already a local player present! remove first!" << endl;// maybe use boWarning() only
-	return;
- }
  if (boGame->playerCount() != 0) {
 	boError() << k_funcinfo << "there are already players in the game!" << endl;
 	return;
  }
  boDebug() << k_funcinfo << endl;
  Player* p = new Player;
+ p->addGameIO(new BosonLocalPlayerInput()); // we can use the RTTI of this to identify the local player
  boGame->bosonAddPlayer(p);
- slotChangeLocalPlayer(p);
+ if (d->mStartup) {
+	// this must be done _now_, we cannot delay it!
+	// -> the newgame widget must know about the local player
+	d->mStartup->setLocalPlayer(p);
+ }
 }
 
+// TODO: when this fails we should go back to the welcome widget!
 void TopWidget::slotGameStarted()
 {
- boDebug() << k_funcinfo << endl;
+ boDebug(270) << k_funcinfo << endl;
  BO_CHECK_NULL_RET(boGame);
  if (boGame->gameStatus() != KGame::Run) {
-	boWarning() << k_funcinfo << "not in Run status" << endl;
+	boWarning(270) << k_funcinfo << "not in Run status" << endl;
 	return;
  }
-// AB: first init map (see slotAssginMap()), THEN init player. we need map for
-// player loading (unit positions, ...)
 
- boDebug() << k_funcinfo << "init player" << endl;
- if (!boGame->localPlayer()) {
-	boError() << k_funcinfo << "NULL local player" << endl;
+ boDebug(270) << k_funcinfo << "init player" << endl;
+ Player* localPlayer = 0;
+ for (unsigned int i = 0; i < boGame->playerCount(); i++) {
+	Player* p = (Player*)boGame->playerList()->at(i);
+	if (!p->isVirtual()) {
+		// a non-virtual player is a player that is running on this host
+		// (either the local player or a computer player - never a
+		// network player)
+		if (p->hasRtti(BosonLocalPlayerInput::LocalPlayerInputRTTI)) {
+			if (p->hasRtti(KGameIO::ComputerIO)) {
+				boError(270) << k_funcinfo << "a player must not be both - local player and computer player!" << endl;
+				return;
+			}
+			if (localPlayer) {
+				boError(270) << k_funcinfo << "two local players found?!" << endl;
+				return;
+			}
+			localPlayer = p;
+
+			// when starting the game, a player must not have a
+			// BosonLocalPlayerInput!
+			KGameIO* io = p->findRttiIO(BosonLocalPlayerInput::LocalPlayerInputRTTI);
+			p->removeGameIO(io);
+		}
+	}
+ }
+
+ if (!localPlayer) {
+	// this can happen in two cases
+	// 1. editor mode
+	// 2. loading a game
+	
+	if (!boGame->gameMode()) {
+		// pick one player for editor mode
+		// (it doesnt matter which)
+		boDebug() << k_funcinfo << "picking a local player for editor mode" << endl;
+		localPlayer = (Player*)boGame->playerList()->at(0);
+	} else {
+		// we are loading a game
+		// we are chosing the first player here - this is
+		// valid as long as we don't support loading network games.
+		// later we may allow selecting a player in the startup widgets
+		//
+		// if we ever do that, we should remove these line here!
+		boDebug() << k_funcinfo << "picking a local player for loading games" << endl;
+		localPlayer = (Player*)boGame->playerList()->at(0);
+	}
+ }
+ if (!localPlayer) {
+	boError(270) << k_funcinfo << "NULL local player" << endl;
 	return;
  }
- changeLocalPlayer(boGame->localPlayer());
+ if (d->mBosonWidget->localPlayer()) {
+	boWarning(270) << k_funcinfo << "localPlayer should ne NULL here!" << endl;
+ }
+ slotChangeLocalPlayer(localPlayer);
  d->mBosonWidget->initPlayer();
 
+ if (d->mBosonWidget->canvas()) {
+	boWarning(270) << k_funcinfo << "BosonWidget::canvas() is non-NULL ("
+			<< d->mBosonWidget->canvas()
+			<< ")! This is very unexpected!" << endl;
+	boWarning(270) << k_funcinfo << "setting it to " << d->mBosonWidget->canvas() << endl;
+
+ }
+ d->mBosonWidget->setCanvas(boGame->canvasNonConst());
+ d->mBosonWidget->initMap();
+
+ d->mDisplayManager->setCanvas(boGame->canvasNonConst());
+
  // now show the bosonwidget and hide the startup widgets.
  mMainDock->setWidget(d->mBosonWidget);
  d->mBosonWidget->show();
@@ -1121,6 +1167,19 @@ void TopWidget::slotNewGame(KCmdLineArgs* args)
  d->mStartup->slotNewGame(args);
 }
 
+void TopWidget::slotLoadFromLog(const QString& logFile)
+{
+ if (!d->mStartup) {
+	boError() << k_funcinfo << "NULL startup widget" << endl;
+	return;
+ }
+ BO_CHECK_NULL_RET(boGame);
+ boDebug() << k_funcinfo << "trying to load from log file " << logFile << endl;
+ d->mStartup->slotNewGame(0);
+ BO_CHECK_NULL_RET(d->mStarting);
+ d->mStarting->setLoadFromLogFile(logFile);
+}
+
 void TopWidget::slotStartingFailed()
 {
  boDebug() << k_funcinfo << endl;
@@ -1156,6 +1215,7 @@ void TopWidget::slotDebugKGame()
  KGamePlayerDebug* player = new KGamePlayerDebug(b);
  player->setBoson(boGame);
 
+#if 0
  if (boGame->playField()) {
 	BosonMap* map = boGame->playField()->map();
 	if (!map) {
@@ -1163,9 +1223,14 @@ void TopWidget::slotDebugKGame()
 		return;
 	}
 	b = dlg->addVBoxPage(i18n("Debug &Cells"));
+
+	// AB: this hardly does anything atm (04/04/23), but it takes a lot of
+	// time and memory to be initialized on big maps (on list item per cell,
+	// on a 500x500 map thats a lot)
 	KGameCellDebug* cells = new KGameCellDebug(b);
 	cells->setMap(map);
  }
+#endif
 
  connect(dlg, SIGNAL(signalRequestIdName(int,bool,QString&)),
 		this, SLOT(slotDebugRequestIdName(int,bool,QString&)));
@@ -1235,3 +1300,8 @@ void TopWidget::slotDebugRequestIdName(int msgid, bool , QString& name)
 // boDebug() << name << endl;
 }
 
+void TopWidget::slotChangeLocalPlayer(Player* p)
+{
+ changeLocalPlayer(p);
+}
+
