@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2002 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2002-2004 The Boson Team (boson-devel@lists.sourceforge.net)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -34,8 +34,10 @@ class Cell;
 class Player;
 class PluginProperties;
 class Boson;
+class BosonItem;
 class BosonWeapon;
 class QPoint;
+class QDomElement;
 
 /**
  * @author Andreas Beckermann <b_mann@gmx.de>
@@ -49,8 +51,10 @@ public:
 		Repair = 2,
 		Harvester = 3,
 		Weapon = 4, // note: this won't end up in Unit::plugin()! weapons are stored separately. also note that rtti==Weapon is *not* unique! they have their own class and rttis - see BosonWeapon
-		Bomb = 5,
-		Mine = 6,
+		Bombing = 5,
+		Mining = 6, // placing mine (the exploding ones)
+		ResourceMine = 7,
+		Refinery = 8,
 
 		PluginEnd // MUST be the last entry!
 	};
@@ -97,6 +101,16 @@ public:
 
 	virtual int pluginType() const = 0;
 
+	/**
+	 * Called when @p item is about to be removed from the game. When your
+	 * plugin stores a pointer to an item (e.g. a unit, such as a pointer to
+	 * a refinery), you should set it at least to NULL now.
+	 *
+	 * Note that at this point @p item has not yet been deleted, but it will
+	 * be soon!
+	 **/
+	virtual void itemRemoved(BosonItem* item) = 0;
+
 	/**
 	 * @param advanceCount See @ref BosonCanvas::slotAdvance. You can use
 	 * this to do expensive calculations only as seldom as possible. Note
@@ -105,6 +119,32 @@ public:
 	 **/
 	virtual void advance(unsigned int advanceCount) = 0;
 
+	/**
+	 * Save the plugin into @p root. You must implement this in derived
+	 * classes, but usually you will simply return true without touching @p
+	 * root.
+	 *
+	 * Note that you are meant to use @ref KGameProperty for most of the
+	 * properties. Don't use saveAsXML() for integer values or so.
+	 *
+	 * You should save e.g. IDs that help you to identify a pointer (e.g.
+	 * the harvester plugin will save the ID of the refinery it is going
+	 * to).
+	 *
+	 * See also @ref loadFromXML.
+	 **/
+	virtual bool saveAsXML(QDomElement& root) const = 0;
+
+	/**
+	 * See also @ref saveAsXML. You should use @ref KGameProperty for most
+	 * properties.
+	 *
+	 * Here you will usually load pointers - e.g. you could save the ID of a
+	 * target in @ref saveAsXML and here you could set the pointer of the
+	 * target.
+	 **/
+	virtual bool loadFromXML(const QDomElement& root) = 0;
+
 private:
 	Unit* mUnit;
 };
@@ -184,9 +224,12 @@ public:
 
 	virtual void advance(unsigned int);
 
+	virtual bool saveAsXML(QDomElement& root) const;
+	virtual bool loadFromXML(const QDomElement& root);
+
+	virtual void itemRemoved(BosonItem*) {}
+
 private:
-//	KGamePropertyList<QPair<ProductionType, unsigned long int> > mProductions;
-	/// TODO: make this KGameProperty
 	QValueList<QPair<ProductionType, unsigned long int> > mProductions;
 	KGameProperty<unsigned int> mProductionState;
 };
@@ -209,7 +252,7 @@ public:
 
 	/**
 	 * Order to repair unit. For a repairyard this means the unit will move
-	 * to the repairyard and once it is in range it'll be repaired. 
+	 * to the repairyard and once it is in range it'll be repaired.
 	 *
 	 * For mobile repair-units this means that the <em>repairing</em> (i.e.
 	 * the one that has this plugin) moves to unit and repairs it.
@@ -230,10 +273,17 @@ public:
 	// does nothing, yet. plugin is experimental anyway.
 	virtual void advance(unsigned int) {}
 
+	virtual bool saveAsXML(QDomElement& root) const;
+	virtual bool loadFromXML(const QDomElement& root);
+
+	virtual void itemRemoved(BosonItem*);
+
 private:
 };
 
-/*
+class ResourceMinePlugin;
+class RefineryPlugin;
+/**
  * @author Andreas Beckermann <b_mann@gmx.de>
  **/
 class HarvesterPlugin : public UnitPlugin
@@ -252,14 +302,8 @@ public:
 	int resourcesY() const { return mResourcesY; }
 	unsigned int resourcesMined() const { return mResourcesMined; }
 
-	void mineAt(const QPoint& pos);
-	void refineAt(Unit* refinery);
-
-	void setRefinery(Unit* refinery);
-
-	bool canMine(Cell* cell) const;
-
-	inline Unit* refinery() const { return mRefinery; }
+	void mineAt(ResourceMinePlugin* resource);
+	void refineAt(RefineryPlugin* refinery);
 
 	/**
 	 * @return PluginProperties::canMineMinerals
@@ -286,6 +330,19 @@ public:
 	 **/
 	unsigned int unloadingSpeed() const;
 
+	virtual bool saveAsXML(QDomElement& root) const;
+	virtual bool loadFromXML(const QDomElement& root);
+
+	virtual void itemRemoved(BosonItem*);
+
+protected:
+	bool isAtResourceMine() const;
+	bool isAtRefinery() const;
+	bool isNextTo(const Unit* unit) const;
+
+	ResourceMinePlugin* findClosestResourceMine() const;
+	RefineryPlugin* findClosestRefinery() const;
+
 private:
 	KGameProperty<int> mResourcesX;
 	KGameProperty<int> mResourcesY;
@@ -293,7 +350,8 @@ private:
 
 	KGameProperty<int> mHarvestingType; // either mining or refining
 
-	Unit* mRefinery; // TODO we need to store this when Unit::save() is called!
+	RefineryPlugin* mRefinery;
+	ResourceMinePlugin* mResourceMine;
 };
 
 /**
@@ -307,16 +365,21 @@ public:
 	BombingPlugin(Unit* owner);
 	~BombingPlugin();
 
-	virtual int pluginType() const { return Bomb; }
+	virtual int pluginType() const { return Bombing; }
 
 	void bomb(int weaponId, float x, float y);
 
 	virtual void advance(unsigned int);
 
+	virtual bool saveAsXML(QDomElement& root) const;
+	virtual bool loadFromXML(const QDomElement& root);
+
+	virtual void itemRemoved(BosonItem*) {}
+
 private:
-	BosonWeapon* mWeapon;
-	int mPosX;
-	int mPosY;
+	BosonWeapon* mWeapon; // FIXME: must be saved on Unit::save()
+	KGameProperty<float> mPosX;
+	KGameProperty<float> mPosY;
 };
 
 /**
@@ -331,15 +394,123 @@ public:
 	MiningPlugin(Unit* owner);
 	~MiningPlugin();
 
-	virtual int pluginType() const { return Mine; }
+	virtual int pluginType() const { return Mining; }
 
 	void mine(int weaponId);
 
 	virtual void advance(unsigned int);
 
+	virtual bool saveAsXML(QDomElement& root) const;
+	virtual bool loadFromXML(const QDomElement& root);
+
+	virtual void itemRemoved(BosonItem*) {}
+
 private:
-	BosonWeapon* mWeapon;
-	int mPlacingCounter;
+	BosonWeapon* mWeapon; // FIXME: must be saved in Unit::save()
+	KGameProperty<int> mPlacingCounter;
+};
+
+/**
+ * @short Plugin for mineral/oil mines
+ * @author Andreas Beckermann <b_mann@gmx.de>
+ **/
+class ResourceMinePlugin : public UnitPlugin
+{
+public:
+	ResourceMinePlugin(Unit* owner);
+	~ResourceMinePlugin();
+
+	virtual int pluginType() const { return ResourceMine; }
+
+	virtual bool loadFromXML(const QDomElement& root);
+	virtual bool saveAsXML(QDomElement& root) const;
+	virtual void advance(unsigned int advanceCount);
+
+	bool isUsableTo(const HarvesterPlugin* harvester) const;
+
+	/**
+	 * @return How much minerals are left here. -1 means unlimited.
+	 **/
+	int minerals() const;
+
+	/**
+	 * @return How much oil is left here. -1 means unlimited.
+	 **/
+	int oil() const;
+
+	/**
+	 * Mine minerals. The amount of @ref minerals is (if limited) reduced
+	 * by the returned value.
+	 * @return The amount of mined minerals
+	 **/
+	unsigned int mineMinerals(const HarvesterPlugin* harvester);
+
+	/**
+	 * Mine oil. The amount of @ref oil is (if limited) reduced
+	 * by the returned value.
+	 * @return The amount of mined oil
+	 **/
+	unsigned int mineOil(const HarvesterPlugin* harvester);
+
+	void setMinerals(int m);
+	void setOil(int m);
+
+	/**
+	 * @return See @ref ResourceMinePropeties::canProvideMinerals
+	 **/
+	bool canProvideMinerals() const;
+
+	/**
+	 * @return See @ref ResourceMinePropeties::canProvideOil
+	 **/
+	bool canProvideOil() const;
+
+	virtual void itemRemoved(BosonItem*);
+
+protected:
+	/**
+	 * @return The amount of resources that @ref mineMinerals or @ref
+	 * mineOil will mine.
+	 **/
+	unsigned int mineStep(const HarvesterPlugin* harvester, int resourcesAvailable) const;
+
+private:
+	KGameProperty<int> mOil;
+	KGameProperty<int> mMinerals;
+};
+
+class RefineryPlugin : public UnitPlugin
+{
+public:
+	RefineryPlugin(Unit* owner);
+	~RefineryPlugin();
+
+	virtual int pluginType() const { return Refinery; }
+
+	virtual bool loadFromXML(const QDomElement& root);
+	virtual bool saveAsXML(QDomElement& root) const;
+	virtual void advance(unsigned int advanceCount);
+
+	bool isUsableTo(const HarvesterPlugin* harvester) const;
+
+	bool canRefineMinerals() const;
+	bool canRefineOil() const;
+
+	/**
+	 * Try to refine @p minerals.
+	 * @return The minerals that got actually refined. The harvester should
+	 * reduce it's resources by exactly that amount only
+	 **/
+	unsigned int refineMinerals(unsigned int minerals);
+
+	/**
+	 * Try to refine @p oil.
+	 * @return The oil that got actually refined. The harvester should
+	 * reduce it's resources by exactly that amount only
+	 **/
+	unsigned int refineOil(unsigned int oil);
+
+	virtual void itemRemoved(BosonItem*);
 };
 
 #endif
